@@ -19,6 +19,8 @@
 #include "../ui/toc.h"
 #include "../ui/sidebar.h"
 #include "../ui/pageview.h"
+#include "../ui/presentationwidget.h"
+#include "../settings.h"
 
 #include "../generators/poppler/config-okular-poppler.h"
 
@@ -26,10 +28,12 @@
 #include <KConfigDialog>
 #include <KParts/OpenUrlArguments>
 
+#include <QApplication>
 #include <QClipboard>
 #include <QMessageBox>
 #include <QPushButton>
 #include <QScrollBar>
+#include <QTabletEvent>
 #include <QTemporaryDir>
 #include <QTextEdit>
 #include <QTreeView>
@@ -42,7 +46,18 @@ class CloseDialogHelper : public QObject
     Q_OBJECT
 
 public:
-    CloseDialogHelper(Okular::Part *p, QDialogButtonBox::StandardButton b) : m_part(p), m_button(b), m_clicked(false)
+    CloseDialogHelper(Okular::Part *p, QDialogButtonBox::StandardButton b) : m_widget(p->widget()), m_button(b), m_clicked(false)
+    {
+        QTimer::singleShot(0, this, &CloseDialogHelper::closeDialog);
+    }
+
+    CloseDialogHelper(QWidget *w, QDialogButtonBox::StandardButton b) : m_widget(w), m_button(b), m_clicked(false)
+    {
+        QTimer::singleShot(0, this, &CloseDialogHelper::closeDialog);
+    }
+
+    // Close a modal dialog, which may not be associated to any other widget
+    CloseDialogHelper(QDialogButtonBox::StandardButton b) : m_widget(nullptr), m_button(b), m_clicked(false)
     {
         QTimer::singleShot(0, this, &CloseDialogHelper::closeDialog);
     }
@@ -55,7 +70,7 @@ public:
 private slots:
     void closeDialog()
     {
-        QDialog *dialog = m_part->widget()->findChild<QDialog*>();
+        QWidget *dialog = ( m_widget ) ? m_widget->findChild<QDialog*>() : qApp->activeModalWidget();
         if (!dialog) {
             QTimer::singleShot(0, this, &CloseDialogHelper::closeDialog);
             return;
@@ -66,7 +81,7 @@ private slots:
     }
 
 private:
-    Okular::Part *m_part;
+    QWidget *m_widget;
     QDialogButtonBox::StandardButton m_button;
     bool m_clicked;
 };
@@ -100,6 +115,7 @@ class PartTest
         void testSaveAsToSymlink();
         void testSaveIsSymlink();
         void testSidebarItemAfterSaving();
+        void testViewModeSavingPerFile();
         void testSaveAsUndoStackAnnotations();
         void testSaveAsUndoStackAnnotations_data();
         void testSaveAsUndoStackForms();
@@ -118,7 +134,9 @@ class PartTest
         void testCrashTextEditDestroy();
         void testAnnotWindow();
         void testAdditionalActionTriggers();
+        void testTypewriterAnnotTool();
         void testJumpToPage();
+        void testTabletProximityBehavior();
 
     private:
         void simulateMouseSelection(double startX, double startY, double endX, double endY, QWidget *target);
@@ -1049,6 +1067,36 @@ void PartTest::testSidebarItemAfterSaving()
     QCOMPARE(currentSidebarItem, part.m_sidebar->currentItem());
 }
 
+void PartTest::testViewModeSavingPerFile()
+{
+    QVariantList dummyArgs;
+    Okular::Part part( nullptr, nullptr, dummyArgs );
+
+    // Open some file
+    QVERIFY( openDocument( &part, QStringLiteral( KDESRCDIR "data/file1.pdf" ) ) );
+
+    // Switch to 'continuous' view mode
+    part.m_pageView->setCapability( Okular::View::ViewCapability::Continuous, QVariant( true ) );
+
+    // Close document
+    part.closeUrl();
+
+    // Open another file
+    QVERIFY( openDocument( &part, QStringLiteral( KDESRCDIR "data/file2.pdf" ) ) );
+
+    // Switch to 'non-continuous' mode
+    part.m_pageView->setCapability( Okular::View::ViewCapability::Continuous, QVariant( false ) );
+
+    // Close that document, too
+    part.closeUrl();
+
+    // Open first document again
+    QVERIFY( openDocument( &part, QStringLiteral( KDESRCDIR "data/file1.pdf" ) ) );
+
+    // If per-file view mode saving works, the view mode should be 'continuous' again.
+    QVERIFY( part.m_pageView->capability( Okular::View::ViewCapability::Continuous).toBool() );
+}
+
 void PartTest::testSaveAsUndoStackAnnotations()
 {
     QFETCH(QString, file);
@@ -1611,8 +1659,8 @@ void PartTest::testAnnotWindow()
     auto widget = win2->window()->childAt(win2->mapTo(win2->window(), QPoint(10,  10)));
     QTest::mouseMove(win2->window(), win2->mapTo(win2->window(), QPoint(10,  10)));
     QTest::mouseClick(widget, Qt::LeftButton, Qt::NoModifier, widget->mapFrom(win2, QPoint(10,  10)));
-    QVERIFY( win1->visibleRegion().rects().count() == 3);
-    QVERIFY( win2->visibleRegion().rects().count() == 4);
+    QVERIFY( win1->visibleRegion().rectCount() == 3);
+    QVERIFY( win2->visibleRegion().rectCount() == 4);
 }
 
 // Helper for testAdditionalActionTriggers
@@ -1753,6 +1801,50 @@ void PartTest::testAdditionalActionTriggers()
     verifyTargetStates( QStringLiteral( "pb" ), fields, false, false, false, __LINE__ );
 }
 
+void PartTest::testTypewriterAnnotTool()
+{
+  Okular::Part part(nullptr, nullptr, QVariantList());
+
+  part.openUrl(QUrl::fromLocalFile(QStringLiteral(KDESRCDIR "data/file1.pdf")));
+
+  part.widget()->show();
+  QVERIFY(QTest::qWaitForWindowExposed(part.widget()));
+
+  const int width = part.m_pageView->horizontalScrollBar()->maximum() +
+                    part.m_pageView->viewport()->width();
+  const int height = part.m_pageView->verticalScrollBar()->maximum() +
+                     part.m_pageView->viewport()->height();
+
+  part.m_document->setViewportPage(0);
+
+  QMetaObject::invokeMethod(part.m_pageView, "slotToggleAnnotator", Q_ARG( bool, true ));
+
+  // Find the button for the TypeWriter annotation
+  QList<QToolButton *> toolbuttonList = part.m_pageView->findChildren<QToolButton *>();
+  auto it = std::find_if( toolbuttonList.begin(),
+                          toolbuttonList.end(),
+                          [](const QToolButton * x)  { return x->toolTip().contains("Typewriter"); } );
+
+  QVERIFY(it != toolbuttonList.end());
+  QToolButton* typewriterButton = *it;
+
+  typewriterButton->click();
+
+  QTest::qWait(1000);  // Wait for the "add new note" dialog to appear
+  CloseDialogHelper closeDialogHelper( QDialogButtonBox::Ok );
+
+  QTest::mouseClick(part.m_pageView->viewport(), Qt::LeftButton, Qt::NoModifier, QPoint(width * 0.5, height * 0.2));
+
+  Annotation* annot = part.m_document->page(0)->annotations().first();
+  TextAnnotation* ta = static_cast<TextAnnotation*>( annot );
+  QVERIFY( annot );
+  QVERIFY( ta );
+  QCOMPARE( annot->subType(), Okular::Annotation::AText );
+  QCOMPARE( annot->style().color(), QColor(255,255,255,0) );
+  QCOMPARE( ta->textType(), Okular::TextAnnotation::InPlace );
+  QCOMPARE( ta->inplaceIntent(), Okular::TextAnnotation::TypeWriter );
+}
+
 void PartTest::testJumpToPage() {
     const QString testFile = QStringLiteral( KDESRCDIR "data/simple-multipage.pdf" );
     const int targetPage = 25;
@@ -1782,6 +1874,76 @@ void PartTest::testJumpToPage() {
     QCOMPARE(part.m_pageView->verticalScrollBar()->value(), pageWithSpaceTop - 4);
 }
 
+void PartTest::testTabletProximityBehavior()
+{
+    QVariantList dummyArgs;
+    Okular::Part part{ nullptr, nullptr, dummyArgs };
+    QVERIFY( openDocument( &part, QStringLiteral( KDESRCDIR "data/file1.pdf" ) ) );
+    part.slotShowPresentation();
+    PresentationWidget *w = part.m_presentationWidget;
+    QVERIFY( w );
+    part.widget()->show();
+
+    // close the KMessageBox "There are two ways of exiting[...]"
+    CloseDialogHelper closeDialogHelper( w, QDialogButtonBox::Ok ); // confirm the "To leave, press ESC"
+
+    QTabletEvent enterProximityEvent{ QEvent::TabletEnterProximity,
+                                      QPoint( 10, 10 ), QPoint( 10, 10 ),
+                                      QTabletEvent::Stylus, QTabletEvent::Pen,
+                                      1., 0, 0, 1., 1., 0,
+                                      Qt::NoModifier, 0, Qt::NoButton, Qt::NoButton };
+    QTabletEvent leaveProximityEvent{ QEvent::TabletLeaveProximity,
+                                      QPoint( 10, 10 ), QPoint( 10, 10 ),
+                                      QTabletEvent::Stylus, QTabletEvent::Pen,
+                                      1., 0, 0, 1., 1., 0,
+                                      Qt::NoModifier, 0, Qt::NoButton, Qt::NoButton };
+
+    // Test with the Okular::Settings::EnumSlidesCursor::Visible setting
+    Okular::Settings::self()->setSlidesCursor( Okular::Settings::EnumSlidesCursor::Visible );
+
+    // Send an enterProximity event
+    qApp->notify( qApp, &enterProximityEvent );
+
+    // The cursor should be a cross-hair
+    QVERIFY( w->cursor().shape() == Qt::CursorShape( Qt::CrossCursor ) );
+
+    // Send a leaveProximity event
+    qApp->notify( qApp, &leaveProximityEvent );
+
+    // After the leaveProximityEvent, the cursor should be an arrow again, because
+    // we have set the slidesCursor mode to 'Visible'
+    QVERIFY( w->cursor().shape() == Qt::CursorShape( Qt::ArrowCursor ) );
+
+
+    // Test with the Okular::Settings::EnumSlidesCursor::Hidden setting
+    Okular::Settings::self()->setSlidesCursor( Okular::Settings::EnumSlidesCursor::Hidden );
+
+    qApp->notify( qApp, &enterProximityEvent );
+    QVERIFY( w->cursor().shape() == Qt::CursorShape( Qt::CrossCursor ) );
+    qApp->notify( qApp, &leaveProximityEvent );
+    QVERIFY( w->cursor().shape() == Qt::CursorShape( Qt::BlankCursor ) );
+
+    // Moving the mouse should not bring the cursor back
+    QTest::mouseMove( w, QPoint( 100, 100 ) );
+    QVERIFY( w->cursor().shape() == Qt::CursorShape( Qt::BlankCursor ) );
+
+
+    // First test with the Okular::Settings::EnumSlidesCursor::HiddenDelay setting
+    Okular::Settings::self()->setSlidesCursor( Okular::Settings::EnumSlidesCursor::HiddenDelay );
+
+    qApp->notify( qApp, &enterProximityEvent );
+    QVERIFY( w->cursor().shape() == Qt::CursorShape( Qt::CrossCursor ) );
+    qApp->notify( qApp, &leaveProximityEvent );
+
+    // After the leaveProximityEvent, the cursor should be blank, because
+    // we have set the slidesCursor mode to 'HiddenDelay'
+    QVERIFY( w->cursor().shape() == Qt::CursorShape( Qt::BlankCursor ) );
+
+    // Moving the mouse should bring the cursor back
+    QTest::mouseMove(w, QPoint( 150, 150 ));
+    QVERIFY( w->cursor().shape() == Qt::CursorShape( Qt::ArrowCursor ) );
+}
+
 } // namespace Okular
 
 int main(int argc, char *argv[])
