@@ -16,10 +16,10 @@
 #include <qimage.h>
 #include <qlist.h>
 #include <qpainter.h>
-#include <QtPrintSupport/QPrinter>
+#include <QPrinter>
 
 #include <kaboutdata.h>
-#include <QtCore/QDebug>
+#include <QDebug>
 #include <KLocalizedString>
 
 #include <core/document.h>
@@ -89,7 +89,7 @@ class TIFFGenerator::Private
 {
     public:
         Private()
-          : tiff( 0 ), dev( 0 ) {}
+          : tiff( nullptr ), dev( nullptr ) {}
 
         TIFF* tiff;
         QByteArray data;
@@ -174,7 +174,7 @@ TIFFGenerator::~TIFFGenerator()
     if ( d->tiff )
     {
         TIFFClose( d->tiff );
-        d->tiff = 0;
+        d->tiff = nullptr;
     }
 
     delete d;
@@ -207,7 +207,7 @@ bool TIFFGenerator::loadTiff( QVector< Okular::Page * > & pagesVector, const cha
     if ( !d->tiff )
     {
         delete d->dev;
-        d->dev = 0;
+        d->dev = nullptr;
         d->data.clear();
         return false;
     }
@@ -223,9 +223,9 @@ bool TIFFGenerator::doCloseDocument()
     if ( d->tiff )
     {
         TIFFClose( d->tiff );
-        d->tiff = 0;
+        d->tiff = nullptr;
         delete d->dev;
-        d->dev = 0;
+        d->dev = nullptr;
         d->data.clear();
         m_pageMapping.clear();
     }
@@ -294,35 +294,35 @@ Okular::DocumentInfo TIFFGenerator::generateDocumentInfo( const QSet<Okular::Doc
 
         if ( keys.contains( Okular::DocumentInfo::Description ) )
         {
-            char* buffer = 0;
+            char* buffer = nullptr;
             TIFFGetField( d->tiff, TIFFTAG_IMAGEDESCRIPTION, &buffer );
             docInfo.set( Okular::DocumentInfo::Description, buffer ? QString::fromLatin1( buffer ) : QString() );
         }
 
         if ( keys.contains( Okular::DocumentInfo::Producer ) )
         {
-            char* buffer = 0;
+            char* buffer = nullptr;
             TIFFGetField( d->tiff, TIFFTAG_SOFTWARE, &buffer );
             docInfo.set( Okular::DocumentInfo::Producer, buffer ? QString::fromLatin1( buffer ) : QString() );
         }
 
         if ( keys.contains( Okular::DocumentInfo::Copyright ) )
         {
-            char* buffer = 0;
+            char* buffer = nullptr;
             TIFFGetField( d->tiff, TIFFTAG_COPYRIGHT, &buffer );
             docInfo.set( Okular::DocumentInfo::Copyright, buffer ? QString::fromLatin1( buffer ) : QString() );
         }
 
         if ( keys.contains( Okular::DocumentInfo::Author ) )
         {
-            char* buffer = 0;
+            char* buffer = nullptr;
             TIFFGetField( d->tiff, TIFFTAG_ARTIST, &buffer );
             docInfo.set( Okular::DocumentInfo::Author, buffer ? QString::fromLatin1( buffer ) : QString() );
         }
 
         if ( keys.contains( Okular::DocumentInfo::CreationDate ) )
         {
-            char* buffer = 0;
+            char* buffer = nullptr;
             TIFFGetField( d->tiff, TIFFTAG_DATETIME, &buffer );
             QDateTime date = convertTIFFDateTime( buffer );
             docInfo.set( Okular::DocumentInfo::CreationDate, date.isValid() ? QLocale().toString( date, QLocale::LongFormat ) : QString() );
@@ -433,7 +433,7 @@ int TIFFGenerator::mapPage( int page ) const
     return it.value();
 }
 
-Q_LOGGING_CATEGORY(OkularTiffDebug, "org.kde.okular.generators.tiff")
+Q_LOGGING_CATEGORY(OkularTiffDebug, "org.kde.okular.generators.tiff", QtWarningMsg)
 
 #include "generator_tiff.moc"
 
